use gtk::glib::clone;
use gtk::prelude::{BoxExt, ButtonExt, GtkWindowExt};
use relm4::gtk::prelude::GridExt;
use relm4::{gtk, ComponentParts, ComponentSender, RelmApp, RelmWidgetExt, SimpleComponent};

struct AppModel {
    counter: u8,
}

#[derive(Debug)]
enum AppInput {
    Increment,
    Decrement,
}

struct AppWidgets {
    label: gtk::Label,
}

impl SimpleComponent for AppModel {
    /// The type of the messages that this component can receive.
    type Input = AppInput;
    /// The type of the messages that this component can send.
    type Output = ();
    /// The type of data with which this component will be initialized.
    type Init = u8;
    /// The root GTK widget that this component will create.
    type Root = gtk::Window;
    /// A data structure that contains the widgets that you will need to update.
    type Widgets = AppWidgets;

    fn init_root() -> Self::Root {
        gtk::Window::builder()
            .title("Simple app")
            .default_width(300)
            .default_height(100)
            .build()
    }

    /// Initialize the UI and model.
    fn init(
        counter: Self::Init,
        window: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> relm4::ComponentParts<Self> {
        relm4_icons::initialize_icons();
        let model = AppModel { counter };

        let layout = gtk::Box::builder()
            .orientation(gtk::Orientation::Horizontal)
            // .spacing(5)
            .build();

        let tab_chooser = gtk::Box::builder()
            .orientation(gtk::Orientation::Vertical)
            .build();
        let main_view = gtk::Box::builder()
            .orientation(gtk::Orientation::Vertical)
            .build();
        let ll = gtk::GridLayout::builder()
            .column_homogeneous(true)
            .row_homogeneous(true)
            .build();

        let mcdu = gtk::Grid::builder().build();
        mcdu.set_column_spacing(5);
        mcdu.set_row_spacing(5);
        // mcdu.insert_row(0);
        let a = gtk::Button::default();
        a.set_label("a");
        let b = gtk::Button::default();
        b.set_label("b");

        let alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";

        let mcdu_width = 6;
        for (i, c) in alphabet.char_indices() {
            let button = gtk::Button::default();
            button.set_label(&c.to_string());

            let column = i % mcdu_width;
            let row = (i - column) / mcdu_width;

            mcdu.attach(&button, column as i32, row as i32, 1, 1);
        }
        // mcdu.attach(&b, 1, 0, 1, 1);
        // mcdu.
        // .orientation(gtk::Orientation::Vertical)
        // .build();

        let inc_button = gtk::Button::with_label("Increment3");
        let dec_button = gtk::Button::with_label("Decrement");

        let x = gtk::Button::default();
        x.set_label("Tab 1");
        // x.set_icon_name("plus");

        tab_chooser.append(&x);

        let label = gtk::Label::new(Some(&format!("Counter: {}", model.counter)));
        label.set_margin_all(5);

        window.set_child(Some(&layout));
        layout.set_margin_all(5);
        layout.append(&tab_chooser);
        layout.append(&main_view);
        layout.append(&mcdu);

        let widgets = AppWidgets { label };

        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, _sender: ComponentSender<Self>) {
        match message {
            AppInput::Increment => {
                self.counter = self.counter.wrapping_add(1);
            }
            AppInput::Decrement => {
                self.counter = self.counter.wrapping_sub(1);
            }
        }
    }

    /// Update the view to represent the updated model.
    fn update_view(&self, widgets: &mut Self::Widgets, _sender: ComponentSender<Self>) {
        widgets
            .label
            .set_label(&format!("Counter: {}", self.counter));
    }
}

fn main() {
    let app = RelmApp::new("relm4.test.simple_manual");
    app.run::<AppModel>(0);
}
